@@ -137,10 +137,12 @@ struct _GcrParserPrivate {
 	GTree *specific_formats;
 	gboolean normal_formats;
 	GPtrArray *passwords;
-	
+
 	GckAttributes *parsed_attrs;
 	const gchar *parsed_desc;
 	gchar *parsed_label;
+	gconstpointer parsing_block;
+	gsize parsing_n_block;
 };
 
 G_DEFINE_TYPE (GcrParser, gcr_parser, G_TYPE_OBJECT);
@@ -233,8 +235,15 @@ parsed_asn1_attribute (GcrParser *self, GNode *asn, const guchar *data, gsize n_
 }
 
 static void
-parsed_clear (GcrParser *self, CK_OBJECT_CLASS klass)
+parsing_begin (GcrParser *self,
+               CK_OBJECT_CLASS klass,
+               gconstpointer block,
+               gsize n_block)
 {
+	g_assert (GCR_IS_PARSER (self));
+	g_assert (block);
+	g_assert (n_block);
+
 	if (self->pv->parsed_attrs)
 		gck_attributes_unref (self->pv->parsed_attrs);
 	if (klass == CKO_PRIVATE_KEY)
@@ -242,10 +251,7 @@ parsed_clear (GcrParser *self, CK_OBJECT_CLASS klass)
 	else 
 		self->pv->parsed_attrs = gck_attributes_new ();
 	gck_attributes_add_ulong (self->pv->parsed_attrs, CKA_CLASS, klass);
-	
-	g_free (self->pv->parsed_label);
-	self->pv->parsed_label = NULL;
-	
+
 	switch (klass) {
 	case CKO_PRIVATE_KEY:
 		self->pv->parsed_desc = _("Private Key");
@@ -260,6 +266,9 @@ parsed_clear (GcrParser *self, CK_OBJECT_CLASS klass)
 		self->pv->parsed_desc = NULL;
 		break;
 	}
+
+	self->pv->parsing_block = block;
+	self->pv->parsing_n_block = n_block;
 }
 
 static void
@@ -277,6 +286,20 @@ parsed_attribute (GcrParser *self, CK_ATTRIBUTE_TYPE type, gconstpointer data, g
 	gck_attributes_add_data (self->pv->parsed_attrs, type, data, n_data);
 }
 
+static void
+parsing_end (GcrParser *self)
+{
+	g_assert (GCR_IS_PARSER (self));
+	self->pv->parsing_block = NULL;
+	self->pv->parsing_n_block = 0;
+	if (self->pv->parsed_attrs)
+		gck_attributes_unref (self->pv->parsed_attrs);
+	self->pv->parsed_attrs = NULL;
+	g_free (self->pv->parsed_label);
+	self->pv->parsed_label = NULL;
+	self->pv->parsed_desc = NULL;
+}
+
 static void
 parsed_ulong (GcrParser *self, CK_ATTRIBUTE_TYPE type, gulong value)
 {
@@ -364,7 +387,7 @@ parse_der_private_key_rsa (GcrParser *self, const guchar *data, gsize n_data)
 	if (!asn)
 		goto done;
 
-	parsed_clear (self, CKO_PRIVATE_KEY);
+	parsing_begin (self, CKO_PRIVATE_KEY, data, n_data);
 	parsed_ulong (self, CKA_KEY_TYPE, CKK_RSA);
 	parsed_boolean (self, CKA_PRIVATE, CK_TRUE);
 	res = GCR_ERROR_FAILURE;
@@ -389,12 +412,13 @@ parse_der_private_key_rsa (GcrParser *self, const guchar *data, gsize n_data)
 	
 	parsed_fire (self);
 	res = SUCCESS;
+	parsing_end (self);
 
 done:
 	egg_asn1x_destroy (asn);
 	if (res == GCR_ERROR_FAILURE)
 		g_message ("invalid RSA key");
-	
+
 	return res;
 }
 
@@ -412,7 +436,7 @@ parse_der_private_key_dsa (GcrParser *self, const guchar *data, gsize n_data)
 	if (!asn)
 		goto done;
 
-	parsed_clear (self, CKO_PRIVATE_KEY);
+	parsing_begin (self, CKO_PRIVATE_KEY, data, n_data);
 	parsed_ulong (self, CKA_KEY_TYPE, CKK_DSA);
 	parsed_boolean (self, CKA_PRIVATE, CK_TRUE);
 	ret = GCR_ERROR_FAILURE;
@@ -425,12 +449,13 @@ parse_der_private_key_dsa (GcrParser *self, const guchar *data, gsize n_data)
 		
 	parsed_fire (self);
 	ret = SUCCESS;
+	parsing_end (self);
 
 done:
 	egg_asn1x_destroy (asn);
 	if (ret == GCR_ERROR_FAILURE) 
 		g_message ("invalid DSA key");
-		
+
 	return ret;
 }
 
@@ -447,7 +472,6 @@ parse_der_private_key_dsa_parts (GcrParser *self, const guchar *keydata, gsize n
 	if (!asn_params || !asn_key)
 		goto done;
 
-	parsed_clear (self, CKO_PRIVATE_KEY);
 	parsed_ulong (self, CKA_KEY_TYPE, CKK_DSA);
 	parsed_boolean (self, CKA_PRIVATE, CK_TRUE);
 	ret = GCR_ERROR_FAILURE;
@@ -466,8 +490,8 @@ done:
 	egg_asn1x_destroy (asn_params);
 	if (ret == GCR_ERROR_FAILURE) 
 		g_message ("invalid DSA key");
-		
-	return ret;	
+
+	return ret;
 }
 
 /* -----------------------------------------------------------------------------
@@ -478,11 +502,11 @@ static gint
 parse_der_private_key (GcrParser *self, const guchar *data, gsize n_data)
 {
 	gint res;
-	
+
 	res = parse_der_private_key_rsa (self, data, n_data);
 	if (res == GCR_ERROR_UNRECOGNIZED)
 		res = parse_der_private_key_dsa (self, data, n_data);
-		
+
 	return res;
 }
 
@@ -533,7 +557,7 @@ parse_der_pkcs8_plain (GcrParser *self, const guchar *data, gsize n_data)
 	ret = SUCCESS;
 	
 done:
-	if (ret == SUCCESS) {		
+	if (ret == SUCCESS) {
 		switch (key_type) {
 		case CKK_RSA:
 			ret = parse_der_private_key_rsa (self, keydata, n_keydata);
@@ -541,18 +565,20 @@ done:
 		case CKK_DSA:
 			/* Try the normal sane format */
 			ret = parse_der_private_key_dsa (self, keydata, n_keydata);
-			
+
+			parsing_begin (self, CKO_PRIVATE_KEY, data, n_data);
 			/* Otherwise try the two part format that everyone seems to like */
 			if (ret == GCR_ERROR_UNRECOGNIZED && params && n_params)
 				ret = parse_der_private_key_dsa_parts (self, keydata, n_keydata, 
 				                                       params, n_params);
+			parsing_end (self);
 			break;
 		default:
 			g_message ("invalid or unsupported key type in PKCS#8 key");
 			ret = GCR_ERROR_UNRECOGNIZED;
 			break;
 		};
-		
+
 	} else if (ret == GCR_ERROR_FAILURE) {
 		g_message ("invalid PKCS#8 key");
 	}
@@ -591,7 +617,7 @@ parse_der_pkcs8_encrypted (GcrParser *self, const guchar *data, gsize n_data)
 
 	params = egg_asn1x_get_raw_element (egg_asn1x_node (asn, "encryptionAlgorithm", "parameters", NULL), &n_params);
 
-	parsed_clear (self, CKO_PRIVATE_KEY);
+	parsing_begin (self, CKO_PRIVATE_KEY, data, n_data);
 
 	/* Loop to try different passwords */                       
 	for (;;) {
@@ -628,6 +654,7 @@ parse_der_pkcs8_encrypted (GcrParser *self, const guchar *data, gsize n_data)
 		
 		/* Try to parse the resulting key */
 		r = parse_der_pkcs8_plain (self, crypted, n_crypted);
+		egg_secure_free (crypted);
 		crypted = NULL;
 		
 		if (r != GCR_ERROR_UNRECOGNIZED) {
@@ -637,7 +664,9 @@ parse_der_pkcs8_encrypted (GcrParser *self, const guchar *data, gsize n_data)
 		
 		/* We assume unrecognized data, is a bad encryption key */	
 	}
-		
+
+	parsing_end (self);
+
 done:
 	if (cih)
 		gcry_cipher_close (cih);
@@ -666,17 +695,20 @@ parse_der_pkcs8 (GcrParser *self, const guchar *data, gsize n_data)
 static gint
 parse_der_certificate (GcrParser *self, const guchar *data, gsize n_data)
 {
+	gchar *name = NULL;
 	GNode *asn;
-	gchar *name;
 
 	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "Certificate", data, n_data);
 	if (asn == NULL)
 		return GCR_ERROR_UNRECOGNIZED;
 
-	parsed_clear (self, CKO_CERTIFICATE);
+	parsing_begin (self, CKO_CERTIFICATE, data, n_data);
+
 	parsed_ulong (self, CKA_CERTIFICATE_TYPE, CKC_X_509);
 
-	name = egg_dn_read_part (egg_asn1x_node (asn, "tbsCertificate", "subject", "rdnSequence", NULL), "CN");
+	if (self->pv->parsed_label == NULL)
+		name = egg_dn_read_part (egg_asn1x_node (asn, "tbsCertificate", "subject", "rdnSequence", NULL), "CN");
+
 	egg_asn1x_destroy (asn);
 
 	if (name != NULL) {
@@ -686,7 +718,8 @@ parse_der_certificate (GcrParser *self, const guchar *data, gsize n_data)
 
 	parsed_attribute (self, CKA_VALUE, data, n_data);
 	parsed_fire (self);
-	
+
+	parsing_end (self);
 	return SUCCESS;
 }
 
@@ -782,6 +815,38 @@ done:
  * PKCS12
  */
 
+static GNode *
+decode_pkcs12_asn1_accepting_invalid_crap (const ASN1_ARRAY_TYPE *defs,
+                                           const gchar *identifier,
+                                           gconstpointer data,
+                                           gsize n_data)
+{
+	GNode *asn;
+
+	/*
+	 * Because PKCS#12 files, the bags specifically, are notorious for
+	 * being crappily constructed and are often break rules such as DER
+	 * sorting order etc.. we parse the DER in a non-strict fashion.
+	 *
+	 * The rules in DER are designed for X.509 certificates, so there is
+	 * only one way to represent a given certificate (although they fail
+	 * at that as well). But with PKCS#12 we don't have such high
+	 * requirements, and we can slack off on our validation.
+	 */
+
+	asn = egg_asn1x_create (defs, identifier);
+	g_return_val_if_fail (asn != NULL, NULL);
+
+	/* Passing FALSE as the strictness argument */
+	if (!egg_asn1x_decode_no_validate (asn, data, n_data) ||
+	    !egg_asn1x_validate (asn, FALSE)) {
+		egg_asn1x_destroy (asn);
+		asn = NULL;
+	}
+
+	return asn;
+}
+
 static gint
 handle_pkcs12_cert_bag (GcrParser *self, const guchar *data, gsize n_data)
 {
@@ -793,7 +858,9 @@ handle_pkcs12_cert_bag (GcrParser *self, const guchar *data, gsize n_data)
 	gint ret;
 
 	ret = GCR_ERROR_UNRECOGNIZED;
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-12-CertBag", data, n_data);
+	asn = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab,
+	                                                 "pkcs-12-CertBag",
+	                                                 data, n_data);
 	if (!asn)
 		goto done;
 
@@ -820,6 +887,41 @@ done:
 	return ret;
 }
 
+static gchar *
+parse_pkcs12_bag_friendly_name (GNode *asn)
+{
+	guint count, i;
+	GQuark oid;
+	GNode *node;
+	gconstpointer element;
+	gsize n_element;
+	GNode *asn_str;
+	gchar *result;
+
+	if (asn == NULL)
+		return NULL;
+
+	count = egg_asn1x_count (asn);
+	for (i = 1; i <= count; i++) {
+		oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, i, "type", NULL));
+		if (oid == GCR_OID_PKCS9_ATTRIBUTE_FRIENDLY) {
+			node = egg_asn1x_node (asn, i, "values", 1, NULL);
+			if (node != NULL) {
+				element = egg_asn1x_get_raw_element (node, &n_element);
+				asn_str = egg_asn1x_create_and_decode (pkix_asn1_tab, "BMPString",
+				                                       element, n_element);
+				if (asn_str) {
+					result = egg_asn1x_get_bmpstring_as_utf8 (asn_str);
+					egg_asn1x_destroy (asn_str);
+					return result;
+				}
+			}
+		}
+	}
+
+	return NULL;
+}
+
 static gint
 handle_pkcs12_bag (GcrParser *self, const guchar *data, gsize n_data)
 {
@@ -828,11 +930,15 @@ handle_pkcs12_bag (GcrParser *self, const guchar *data, gsize n_data)
 	guint count = 0;
 	GQuark oid;
 	const guchar *element;
+	gchar *friendly;
 	gsize n_element;
+	guint i;
 
 	ret = GCR_ERROR_UNRECOGNIZED;
 
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-12-SafeContents", data, n_data);
+	asn = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab,
+	                                                 "pkcs-12-SafeContents",
+	                                                 data, n_data);
 	if (!asn)
 		goto done;
 
@@ -844,23 +950,21 @@ handle_pkcs12_bag (GcrParser *self, const guchar *data, gsize n_data)
 	/* 
 	 * Now inside each bag are multiple elements. Who comes up 
 	 * with this stuff?
-	 * 
-	 * But this is where we draw the line. We only support one
-	 * element per bag, not multiple elements, not strange
-	 * nested bags, not fairy queens with magical wands in bags...
-	 * 
-	 * Just one element per bag.
 	 */
-	if (count >= 1) {
+	for (i = 1; i <= count; i++) {
 
-		oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, 1, "bagId", NULL));
+		oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, i, "bagId", NULL));
 		if (!oid)
 			goto done;
 
-		element = egg_asn1x_get_raw_element (egg_asn1x_node (asn, 1, "bagValue", NULL), &n_element);
+		element = egg_asn1x_get_raw_element (egg_asn1x_node (asn, i, "bagValue", NULL), &n_element);
 		if (!element)
 			goto done;
 
+		friendly = parse_pkcs12_bag_friendly_name (egg_asn1x_node (asn, i, "bagAttributes", NULL));
+		if (friendly != NULL)
+			parsed_label (self, friendly);
+
 		/* A normal unencrypted key */
 		if (oid == GCR_OID_PKCS12_BAG_PKCS8_KEY) {
 			r = parse_der_pkcs8_plain (self, element, n_element);
@@ -877,8 +981,13 @@ handle_pkcs12_bag (GcrParser *self, const guchar *data, gsize n_data)
 		} else {
 			r = GCR_ERROR_UNRECOGNIZED;
 		}
-		 
-		if (r == GCR_ERROR_FAILURE || r == GCR_ERROR_CANCELLED) {
+
+		if (friendly != NULL)
+			parsed_label (self, NULL);
+
+		if (r == GCR_ERROR_FAILURE ||
+		    r == GCR_ERROR_CANCELLED ||
+		    r == GCR_ERROR_LOCKED) {
 			ret = r;
 			goto done;
 		}
@@ -908,7 +1017,9 @@ handle_pkcs12_encrypted_bag (GcrParser *self, const guchar *data, gsize n_data)
 
 	ret = GCR_ERROR_UNRECOGNIZED;
 
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-7-EncryptedData", data, n_data);
+	asn = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab,
+	                                                 "pkcs-7-EncryptedData",
+	                                                 data, n_data);
 	if (!asn)
 		goto done;
 
@@ -923,8 +1034,6 @@ handle_pkcs12_encrypted_bag (GcrParser *self, const guchar *data, gsize n_data)
 	if (!params)
 		goto done;
 
-	parsed_clear (self, 0);
-
 	/* Loop to try different passwords */
 	for (;;) {
 		
@@ -973,7 +1082,7 @@ handle_pkcs12_encrypted_bag (GcrParser *self, const guchar *data, gsize n_data)
 		
 		/* We assume unrecognized data is a bad encryption key */	
 	}
-		
+
 done:
 	if (cih)
 		gcry_cipher_close (cih);
@@ -997,7 +1106,9 @@ handle_pkcs12_safe (GcrParser *self, const guchar *data, gsize n_data)
 
 	ret = GCR_ERROR_UNRECOGNIZED;
 
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-12-AuthenticatedSafe", data, n_data);
+	asn = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab,
+	                                                 "pkcs-12-AuthenticatedSafe",
+	                                                 data, n_data);
 	if (!asn)
 		goto done;
 
@@ -1026,7 +1137,9 @@ handle_pkcs12_safe (GcrParser *self, const guchar *data, gsize n_data)
 		if (oid == GCR_OID_PKCS7_DATA) {
 
 			egg_asn1x_destroy (asn_content);
-			asn_content = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-7-Data", bag, n_bag);
+			asn_content = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab,
+			                                                         "pkcs-7-Data",
+			                                                         bag, n_bag);
 			if (!asn_content)
 				goto done;
 
@@ -1046,8 +1159,10 @@ handle_pkcs12_safe (GcrParser *self, const guchar *data, gsize n_data)
 			g_warning ("unrecognized type of safe content in pkcs12: %s", g_quark_to_string (oid));
 			r = GCR_ERROR_UNRECOGNIZED;
 		}
-		
-		if (r == GCR_ERROR_FAILURE || r == GCR_ERROR_CANCELLED) {
+
+		if (r == GCR_ERROR_FAILURE ||
+		    r == GCR_ERROR_CANCELLED ||
+		    r == GCR_ERROR_LOCKED) {
 			ret = r;
 			goto done;
 		}
@@ -1062,6 +1177,94 @@ done:
 	return ret;
 }
 
+static gint
+verify_pkcs12_safe (GcrParser *self,
+                    GNode *asn,
+                    gconstpointer content,
+                    gsize n_content)
+{
+	PasswordState pstate = PASSWORD_STATE_INIT;
+	const gchar *password;
+	gcry_md_hd_t mdh = NULL;
+	const guchar *mac_digest;
+	gsize mac_len;
+	guchar *digest = NULL;
+	gsize n_digest;
+	GQuark algorithm;
+	GNode *mac_data;
+	gconstpointer params;
+	gsize n_params;
+	int ret, r;
+
+	ret = GCR_ERROR_FAILURE;
+
+	/*
+	 * The MAC is optional (and outside the encryption no less). I wonder
+	 * what the designers (ha) of PKCS#12 were trying to achieve
+	 */
+
+	mac_data = egg_asn1x_node (asn, "macData", NULL);
+	if (mac_data == NULL)
+		return SUCCESS;
+
+	algorithm = egg_asn1x_get_oid_as_quark (egg_asn1x_node (mac_data, "mac",
+	                                        "digestAlgorithm", "algorithm", NULL));
+	if (!algorithm)
+		goto done;
+
+	params = egg_asn1x_get_raw_element (mac_data, &n_params);
+	if (!params)
+		goto done;
+
+	digest = egg_asn1x_get_string_as_raw (egg_asn1x_node (mac_data, "mac", "digest", NULL), NULL, &n_digest);
+	if (!digest)
+		goto done;
+
+	/* Loop to try different passwords */
+	for (;;) {
+		g_assert (mdh == NULL);
+
+		r = enum_next_password (self, &pstate, &password);
+		if (r != SUCCESS) {
+			ret = r;
+			goto done;
+		}
+
+		/* Parse the encryption stuff into a cipher. */
+		if (!egg_symkey_read_mac (algorithm, password, -1, params, n_params,
+		                          &mdh, &mac_len)) {
+			ret = GCR_ERROR_FAILURE;
+			goto done;
+		}
+
+		/* If not the right length, then that's really broken */
+		if (mac_len != n_digest) {
+			r = GCR_ERROR_FAILURE;
+
+		} else {
+			gcry_md_write (mdh, content, n_content);
+			mac_digest = gcry_md_read (mdh, 0);
+			g_return_val_if_fail (mac_digest, GCR_ERROR_FAILURE);
+			r = memcmp (mac_digest, digest, n_digest) == 0 ? SUCCESS : GCR_ERROR_LOCKED;
+		}
+
+		gcry_md_close (mdh);
+		mdh = NULL;
+
+		if (r != GCR_ERROR_LOCKED) {
+			ret = r;
+			break;
+		}
+	}
+
+done:
+	if (mdh)
+		gcry_md_close (mdh);
+	g_free (digest);
+	return ret;
+
+}
+
 static gint
 parse_der_pkcs12 (GcrParser *self, const guchar *data, gsize n_data)
 {
@@ -1075,11 +1278,12 @@ parse_der_pkcs12 (GcrParser *self, const guchar *data, gsize n_data)
 
 	ret = GCR_ERROR_UNRECOGNIZED;
 
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-12-PFX", data, n_data);
+	asn = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab, "pkcs-12-PFX",
+	                                                 data, n_data);
 	if (!asn)
 		goto done;
 
-	ret = GCR_ERROR_FAILURE;
+	parsing_begin (self, 0, data, n_data);
 
 	oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, "authSafe", "contentType", NULL));
 	if (!oid)
@@ -1095,7 +1299,8 @@ parse_der_pkcs12 (GcrParser *self, const guchar *data, gsize n_data)
 	if (!element)
 		goto done;
 
-	asn_content = egg_asn1x_create_and_decode (pkix_asn1_tab, "pkcs-7-Data", element, n_element);
+	asn_content = decode_pkcs12_asn1_accepting_invalid_crap (pkix_asn1_tab, "pkcs-7-Data",
+	                                                         element, n_element);
 	if (!asn_content)
 		goto done;
 
@@ -1103,7 +1308,11 @@ parse_der_pkcs12 (GcrParser *self, const guchar *data, gsize n_data)
 	if (!content)
 		goto done;
 
-	ret = handle_pkcs12_safe (self, content, n_content);
+	ret = verify_pkcs12_safe (self, asn, content, n_content);
+	if (ret == SUCCESS)
+		ret = handle_pkcs12_safe (self, content, n_content);
+
+	parsing_end (self);
 
 done:
 	g_free (content);
@@ -1202,25 +1411,25 @@ handle_encrypted_pem (GcrParser *self, GQuark type, gint subformat,
 		g_message ("missing encryption header");
 		return GCR_ERROR_FAILURE;
 	}
-	
-	/* Fill in information necessary for prompting */
-	parsed_clear (self, pem_type_to_class (type));
-	
+
+	res = GCR_ERROR_FAILURE;
 	for (;;) {
 
 		res = enum_next_password (self, &pstate, &password);
 		if (res != SUCCESS)
-			return res;
-		
+			break;
+
 		decrypted = NULL;
 		n_decrypted = 0;
 		
 		/* Decrypt, this will result in garble if invalid password */	
 		ret = egg_openssl_decrypt_block (val, password, -1, data, n_data, 
 		                                 &decrypted, &n_decrypted);
-		if (!ret)
-			return GCR_ERROR_FAILURE;
-			
+		if (!ret) {
+			res = GCR_ERROR_FAILURE;
+			break;
+		}
+
 		g_assert (decrypted);
 		
 		/* Unpad the DER data */
@@ -1234,10 +1443,10 @@ handle_encrypted_pem (GcrParser *self, GQuark type, gint subformat,
 
 		/* Unrecognized is a bad password */
 		if (res != GCR_ERROR_UNRECOGNIZED)
-			return res;		
+			break;
 	}
-	
-	return GCR_ERROR_FAILURE;
+
+	return res;
 }
 
 typedef struct {
@@ -1247,8 +1456,13 @@ typedef struct {
 } HandlePemArgs;
 
 static void
-handle_pem_data (GQuark type, const guchar *data, gsize n_data,
-                 GHashTable *headers, gpointer user_data)
+handle_pem_data (GQuark type,
+                 const guchar *data,
+                 gsize n_data,
+                 const gchar *outer,
+                 gsize n_outer,
+                 GHashTable *headers,
+                 gpointer user_data)
 {
 	HandlePemArgs *args = (HandlePemArgs*)user_data;
 	gint res = GCR_ERROR_FAILURE;
@@ -1258,21 +1472,26 @@ handle_pem_data (GQuark type, const guchar *data, gsize n_data,
 	/* Something already failed to parse */
 	if (args->result == GCR_ERROR_FAILURE)
 		return;
-	
+
+	/* Fill in information necessary for prompting */
+	parsing_begin (args->parser, pem_type_to_class (type), outer, n_outer);
+
 	/* See if it's encrypted PEM all openssl like*/
 	if (headers) {
 		val = g_hash_table_lookup (headers, "Proc-Type");
 		if (val && strcmp (val, "4,ENCRYPTED") == 0) 
 			encrypted = TRUE;
 	}
-	
+
 	if (encrypted)
 		res = handle_encrypted_pem (args->parser, type, args->subformat,  
 		                            headers, data, n_data); 
 	else
 		res = handle_plain_pem (args->parser, type, args->subformat, 
 		                        data, n_data);
-	
+
+	parsing_end (args->parser);
+
 	if (res != GCR_ERROR_UNRECOGNIZED) {
 		if (args->result == GCR_ERROR_UNRECOGNIZED)
 			args->result = res;
@@ -1671,26 +1890,28 @@ gcr_parser_parse_data (GcrParser *self, gconstpointer data,
                        gsize n_data, GError **error)
 {
 	ForeachArgs args = { self, data, n_data, GCR_ERROR_UNRECOGNIZED };
-	const gchar *message;
+	const gchar *message = NULL;
 	gint i;
 	
 	g_return_val_if_fail (GCR_IS_PARSER (self), FALSE);
 	g_return_val_if_fail (data || !n_data, FALSE);
 	g_return_val_if_fail (!error || !*error, FALSE);
 
-	/* Just the specific formats requested */
-	if (self->pv->specific_formats) { 
-		g_tree_foreach (self->pv->specific_formats, parser_format_foreach, &args);
-		
-	/* All the 'normal' formats */
-	} else if (self->pv->normal_formats) {
-		for (i = 0; i < G_N_ELEMENTS (parser_normal); ++i) {
-			if (parser_format_foreach ((gpointer)(parser_normal + i), 
-			                           (gpointer)(parser_normal + i), &args))
-				break;
+	if (data && n_data) {
+		/* Just the specific formats requested */
+		if (self->pv->specific_formats) {
+			g_tree_foreach (self->pv->specific_formats, parser_format_foreach, &args);
+
+		/* All the 'normal' formats */
+		} else if (self->pv->normal_formats) {
+			for (i = 0; i < G_N_ELEMENTS (parser_normal); ++i) {
+				if (parser_format_foreach ((gpointer)(parser_normal + i),
+							   (gpointer)(parser_normal + i), &args))
+					break;
+			}
 		}
 	}
-	
+
 	switch (args.result) {
 	case SUCCESS:
 		return TRUE;
@@ -1725,28 +1946,26 @@ gcr_parser_parse_data (GcrParser *self, gconstpointer data,
 void
 gcr_parser_format_enable (GcrParser *self, gint format_id)
 {
-	ParserFormat *format;
+	const ParserFormat *format;
+	guint i;
 
 	g_return_if_fail (GCR_IS_PARSER (self));
 
-	if (format_id == -1) {
-		if (self->pv->specific_formats)
-			g_tree_destroy (self->pv->specific_formats);
-		self->pv->specific_formats = NULL;
-		self->pv->normal_formats = TRUE;
-		return;
-	}
-
-	format = parser_format_lookup (format_id);
-	g_return_if_fail (format);
-
-	if (!self->pv->specific_formats) {
-		if (self->pv->normal_formats)
-			return;
+	if (!self->pv->specific_formats)
 		self->pv->specific_formats = g_tree_new (compare_pointers);
-	}
 
-	g_tree_insert (self->pv->specific_formats, format, format);
+	if (format_id != -1) {
+		format = parser_format_lookup (format_id);
+		g_return_if_fail (format);
+		g_tree_insert (self->pv->specific_formats,
+		               (gpointer)format, (gpointer)format);
+	} else {
+		for (i = 0; i < G_N_ELEMENTS (parser_formats); i++) {
+			format = &parser_formats[i];
+			g_tree_insert (self->pv->specific_formats, (gpointer)format,
+			               (gpointer)format);
+		}
+	}
 }
 
 /**
@@ -1847,6 +2066,27 @@ gcr_parser_get_parsed_label (GcrParser *self)
 	return self->pv->parsed_label;
 }
 
+/**
+ * gcr_parser_get_parsed_block:
+ * @self: a parser
+ * @n_block: a location to place the size of the block
+ *
+ * Get the raw data block that represents this parsed object. This is only
+ * valid during the GcrParser::parsed signal.
+ *
+ * Returns: The raw data block of the currently parsed item. The value is
+ *      owned by the parser and should not be freed.
+ */
+gconstpointer
+gcr_parser_get_parsed_block (GcrParser *self,
+                             gsize *n_block)
+{
+	g_return_val_if_fail (GCR_IS_PARSER (self), NULL);
+	g_return_val_if_fail (n_block, NULL);
+	*n_block = self->pv->parsing_n_block;
+	return self->pv->parsing_block;
+}
+
 /* ---------------------------------------------------------------------------------
  * STREAM PARSING
  */
